#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# rasipfon.py (open with python3)

#import sys
import os
import queue
import threading
from time import sleep

from configparser import SafeConfigParser # ConfigParser py2, configparser py3 
from rasipfon_audio import Audio
from rasipfon_gpio import RotaryDial
from rasipfon_linphone_wrapper import Wrapper
from threading import Timer

callback_queue = queue.Queue()

class rasipfon:
    dial_number = ""  # Number to be dialed
    offHook = False   # On/off hook state
    offHookTimeoutTimer = None   # Off hook timeout
    callTimer = None
    RotaryDial = None
    SipClient = None

    def __init__(self):
        #sleep(60)
        print ("---starting---")
        self.parser = SafeConfigParser() # get data from config file
        self.parser.read('rasipfon.conf')

        #signal.signal(signal.SIGINT, self.OnSignal)  ?

        self.Audio = Audio(self.parser)  # Ring tone
        self.Audio.playfile(self.parser.get('soundfiles','startup'))

        self.RotaryDial = RotaryDial() # Rotary dial
        self.RotaryDial.RegisterCallback(NumberCallback = self.GotDigit, OffHookCallback = self.OffHook, OnHookCallback = self.OnHook, OnVerifyHook = self.OnVerifyHook, PushbuttonCallback = self.PushbuttonPressed)

        self.SipClient = Wrapper()
        self.SipClient.StartLinphone()
        self.SipClient.SipRegister(self.parser.get('sip','username'), self.parser.get('sip','hostname'), self.parser.get('sip','password'))
        self.SipClient.RegisterCallbacks(OnIncomingCall = self.OnIncomingCall, OnOutgoingCall = self.OnOutgoingCall, OnRemoteHungupCall = self.OnRemoteHungupCall, OnSelfHungupCall = self.OnSelfHungupCall)

        self.SipClient.start()   # Start SipClient thread

        input("---waiting---\n") # waits on input from terminal to stop the program
        self.Audio.playfile(self.parser.get('soundfiles','shutdown'))
        self.SipClient.StopLinphone()
        os._exit(0) 

    def PushbuttonPressed(self):
        print ("---pushbutton pressed---")
        self.Audio.playfile(self.parser.get('soundfiles','shutdown'))
        sleep(10)
        os.system("sudo shutdown now")

    def OnHook(self):
        print ("---on hook---")
        self.offHook = False
        self.Audio.stophandset()
        # Hang up calls
        if self.SipClient is not None:
            self.SipClient.SipHangup()

    def OffHook(self):
        print ("---off hook---")
        self.offHook = True
        self.dial_number = ""         # Reset current number when off hook
        #self.offHookTimeoutTimer = Timer(5, self.OnOffHookTimeout)
        #self.offHookTimeoutTimer.start()

        # TODO: State for ringing, don't play tone if ringing :P
        print ("---Try to start dialtone---")
        self.Audio.starthandset(self.parser.get('soundfiles','dialtone'))
        self.Audio.stopring()
        if self.SipClient is not None:
            self.SipClient.SipAnswer()

    def OnVerifyHook(self, state):
        if not state:
            self.offHook = False
            self.Audio.stophandset()

    def OnIncomingCall(self):
        print ('---incoming call---')
        self.Audio.startring()

    def OnOutgoingCall(self):
        print ('---outgoing call---')

    def OnRemoteHungupCall(self):
        print ('---hungup---  ---remote disconnected the call---')
        self.Audio.stopring()
        # Now we want to play busy-tone..
        self.Audio.starthandset(self.parser.get('soundfiles','busytone'))

    def OnSelfHungupCall(self):
#        self.Audio.stop()
        print ('---hungup---  ---local disconnected the call---')

    def OnOffHookTimeout(self):
        print ('---offhook timeout___')
        self.Audio.stophandset()
        self.Audio.starthandset(self.parser.get('soundfiles','timeout'))

    def GotDigit(self, digit):
        print ('---got digit: %s---' % digit)
        if self.callTimer is not None:
            self.callTimer.cancel() 
        self.Audio.stophandset()
        self.dial_number += str(digit)
        print ('---dialing number is: %s---' % self.dial_number)
        self.callTimer = Timer(4, self.Call)
        self.callTimer.start()


        # Shutdown command, since our filesystem isn't read only (yet?)
        # This hopefully prevents dataloss.
        # TODO: stop rebooting..
        if self.dial_number == "0666":
            self.Audio.playfile(self.parser.get('soundfiles','shutdown'))
            os.system("halt")

        if len(self.dial_number) == 9:
            if self.offHook:
                print ('--- dialing number: %s---' % self.dial_number)
                self.SipClient.SipCall(self.dial_number)
                self.dial_number = ""

    def Call(self):
        if self.offHook:
            print ('--- dialing number: %s---' % self.dial_number)
            self.SipClient.SipCall(self.dial_number)
            self.dial_number = ""

    def OnSignal(self, signal, frame):
        print ('---Shutting down on %s---' % signal)
        self.RotaryDial.StopVerifyHook()
        self.SipClient.StopLinphone()
        sys.exit(0)

def main():
    rasip = rasipfon()
#    os._exit(0) 
#    exit()

if __name__ == "__main__":
    main()

