#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# rasipfon_audio.py

from threading import Timer
import alsaaudio
import wave
import smbus
from time import sleep, time

headsetvolume = 70
ringvolume = 100

class Audio:    
    shouldring = 0
    ringstart = 0
    shouldplayhandset = 0
    ringtone = None
    ringfile = None
    handsetfile = None
    timerHandset = None
    port = 1                # i2c port = 1 for rev 2
    ampAddr = 0x58          # TPA2016 address

    def __init__(self,parser):
        self.parser = parser
        bus = smbus.SMBus(self.port) #i2c
        td = [0x00, 0x80, 0x00, 0x00, 0x00, 0x1E, 0x80, 0xC0]
        bus.write_i2c_block_data(self.ampAddr, 0, td) # set gain from i2c chip to max        

    def startring(self):
        self.shouldring = 1
        self.ringtone = Timer(0, self.doring)
        self.ringtone.start()
        self.ringstart = time()

    def stopring(self):
        self.shouldring = 0
        if self.ringtone is not None:
            print("canceling the ring timer")
            self.ringtone.cancel()

    def starthandset(self, file):
        self.shouldplayhandset = 1
        self.handsetfile = file
        if self.timerHandset is not None:
            print ('---handset already playing?---  (mod ringtone)')
            return
        self.timerHandset = Timer(0, self.playhandset)
        self.timerHandset.start()

    def stophandset(self):
        self.shouldplayhandset = 0
        if self.timerHandset is not None:
            self.timerHandset.cancel()
            self.timerHandset = None

    def playhandset(self):
        print ('---starting headset tone---')
        wav = wave.open(self.handsetfile)
        aUsb = alsaaudio.PCM(cardindex=1)
        aUsb_mix = alsaaudio.Mixer(control='Speaker',cardindex=1)
        aUsb_mix.setvolume(headsetvolume)
        aUsb.setchannels(wav.getnchannels())
        aUsb.setrate(wav.getframerate())
        aUsb.setperiodsize(320)
        data = wav.readframes(320)
        while data and self.shouldplayhandset:
            aUsb.write(data)
            data = wav.readframes(320)
        wav.rewind()
        wav.close()

    def playfile(self, file):
        wav = wave.open(file)
        self.aRpi = alsaaudio.PCM(cardindex=0)
        self.aRpi_mix = alsaaudio.Mixer(control='PCM',cardindex=0)
        self.aRpi_mix.setvolume(100)
        self.aRpi.setchannels(wav.getnchannels())
        self.aRpi.setrate(wav.getframerate())
        self.aRpi.setperiodsize(320)
        data = wav.readframes(320)
        while data:
            self.aRpi.write(data)
            data = wav.readframes(320)
        wav.rewind()
        wav.close()

    def doring(self):
        if self.ringfile is not None:
            self.ringfile.rewind()
        else:
            file = self.parser.get('soundfiles','ringtone')
            self.ringfile = wave.open(file, 'rb')
            self.aRpi = alsaaudio.PCM(cardindex=0)
            self.aRpi_mix = alsaaudio.Mixer(control='PCM',cardindex=0)
            self.aRpi_mix.setvolume(ringvolume)
            self.aRpi.setchannels(self.ringfile.getnchannels())
            self.aRpi.setrate(self.ringfile.getframerate())
            self.aRpi.setperiodsize(320)
        while self.shouldring:
            data = self.ringfile.readframes(320)
            while data:
                self.aRpi.write(data)
                data = self.ringfile.readframes(320)
            self.ringfile.rewind()
            sleep(2)
            if time() - 60 > self.ringstart:
                self.stop()
