#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Rotary Dial Parser
# Albiswerk Zürich AW. Ms. 54 S36d
# GND on AW7 and AW8
# 
# Expects the following hardware rules: 1 is 1 pulse, 9 is 9 pulses, 0 is 10 pulses

import RPi.GPIO as GPIO
from threading import Timer
import time
import os

class RotaryDial:
    pin_rotary = 25 # AW9 connected to BCM GPIO 25
    pin_onhook = 24 # AW6 connected to BCM GPIO 24
    pin_pushb  = 23 # AW4 connected to BCM GPIO 23

    digit_timeout = 0.6    # Wait 600ms, for rotation to be done 
    current_digit = 0    # We keep a counter to count each pulse.
    number_timeout = None    # Simple timer for handling the number callback
    last_input = 0
    onhook_timer = None    # Timer to ensure we're on hook
    should_verify_hook = True

    def __init__(self):
        GPIO.setmode(GPIO.BCM)        # Set GPIO mode to Broadcom SOC numbering
        GPIO.setup(self.pin_rotary, GPIO.IN)        # detect rotary movements
        GPIO.add_event_detect(self.pin_rotary, GPIO.BOTH, callback = self.NumberCounter)
        GPIO.setup(self.pin_pushb, GPIO.IN)         #detect pushButton events
        GPIO.add_event_detect(self.pin_pushb, GPIO.BOTH, callback = self.PushbuttonEvent, bouncetime=100)        
        GPIO.setup(self.pin_onhook, GPIO.IN)        # detect hook switch events
        GPIO.add_event_detect(self.pin_onhook, GPIO.BOTH, callback = self.HookEvent, bouncetime=100)
        self.onhook_timer = Timer(2, self.verifyHook)
        self.onhook_timer.start()

    def PushbuttonEvent(self, channel):
        self.PushbuttonCallback()
#        os.system("sudo shutdown now")

    def NumberCounter(self, channel):
        ''' Handle counting of rotary movements and respond with digit after timeout'''
        input = GPIO.input(self.pin_rotary)
        #print ('---INPUT %s (%s)---' % (input, channel))
        if input and not self.last_input:
            self.current_digit += 1
            if self.number_timeout is not None:
                self.number_timeout.cancel()
            self.number_timeout = Timer(self.digit_timeout, self.FoundNumber)
            self.number_timeout.start()
        self.last_input = input
   #     time.sleep(0.002)

    def HookEvent(self, channel):
        ''' Wrapper around the off/on hook event '''
        input = GPIO.input(self.pin_onhook)
        if not input:
            self.hook_state = 1
            self.OffHookCallback()
        else:
            self.hook_state = 0
            self.OnHookCallback()

    def StopVerifyHook(self):
        self.should_verify_hook = False

    def verifyHook(self):
        while self.should_verify_hook:
            state = not GPIO.input(self.pin_onhook) 
            self.OnVerifyHook(state)
            time.sleep(1)

    def FoundNumber(self):
        ''' When the rotary movement has timed out, we callback with the final digit'''
        if self.current_digit == 10:
            self.current_digit = 0
        self.NumberCallback(self.current_digit)
        self.current_digit = 0

    def RegisterCallback(self, NumberCallback, OffHookCallback, OnHookCallback, OnVerifyHook, PushbuttonCallback):
        '''  Handles the callbacks we're supplying '''
        self.NumberCallback = NumberCallback
        self.OffHookCallback = OffHookCallback
        self.OnHookCallback = OnHookCallback
        self.OnVerifyHook = OnVerifyHook
        self.PushbuttonCallback = PushbuttonCallback


        input = GPIO.input(self.pin_onhook)
        if not input:
            self.OffHookCallback()
        else:
            self.OnHookCallback()
